// Generated by CoffeeScript 1.9.1
var DefaultFuryEmitter, NEW_VERSION_REGEXP, PARSER_TIMEOUT, STRICT_OPTIONS, apiBlueprintAdapter, apiaryBlueprintAdapter, apiaryBlueprintParser, base, countLines, getAstSync, getLocalAst, protagonist;

apiaryBlueprintParser = require('apiary-blueprint-parser');

protagonist = require('protagonist');

DefaultFuryEmitter = require('../fury-emitter');

apiBlueprintAdapter = require('./api-blueprint-adapter');

apiaryBlueprintAdapter = require('./apiary-blueprint-adapter');

NEW_VERSION_REGEXP = new RegExp('^(((VERSION:( |\t)2)|(FORMAT:( |\t)(X-)?1A))\n)', 'i');

STRICT_OPTIONS = {
  requireBlueprintName: true
};

if ((base = process.env).PARSER_TIMEOUT == null) {
  base.PARSER_TIMEOUT = 10000;
}

PARSER_TIMEOUT = parseInt(process.env.PARSER_TIMEOUT, 10);

console.log('\nTIMEOUT: ' + PARSER_TIMEOUT + '\n\n');

countLines = function(code, index) {
  var excerpt;
  if (index > 0) {
    excerpt = code.substr(0, index);
    return excerpt.split(/\r\n|\r|\n/).length;
  } else {
    return 1;
  }
};

getLocalAst = function(arg, cb) {
  var apiaryAst, blueprintId, code, emitter, err, options, parseHasFinished, sourcemap, t, timeoutHandler, timeoutTimer;
  code = arg.code, blueprintId = arg.blueprintId, sourcemap = arg.sourcemap, emitter = arg.emitter;
  if (blueprintId == null) {
    blueprintId = '';
  }
  if (emitter == null) {
    emitter = DefaultFuryEmitter();
  }
  if (code.match(NEW_VERSION_REGEXP)) {
    parseHasFinished = false;
    timeoutHandler = function() {
      var err, errorMessage, errorType;
      if (parseHasFinished) {
        return;
      }
      parseHasFinished = true;
      errorType = 'PARSE_TIMEOUT';
      errorMessage = 'Parsing of \'#{blueprintId}\' has timed out';
      emitter.emit('error', errorType + ' ' + errorMessage);
      err = new Error(errorMessage);
      err.errType = errorType;
      err.message = ''.concat(err.toString());
      err.description = ''.concat(err.toString());
      err.location = [
        {
          index: 1,
          length: 0
        }
      ];
      err.line = 1;
      return cb(err);
    };
    options = STRICT_OPTIONS;
    if (sourcemap) {
      options['exportSourcemap'] = true;
    }
    timeoutTimer = setTimeout(timeoutHandler, PARSER_TIMEOUT);
    t = process.hrtime();
    return protagonist.parse(code, options, function(err, result) {
      var execTime, ref;
      execTime = process.hrtime(t);
      execTime = execTime[0] + execTime[1] * 10e-9;
      emitter.emit('metric', 'snowcrashParse', execTime);
      clearTimeout(timeoutTimer);
      if (parseHasFinished) {
        return;
      }
      parseHasFinished = true;
      if (err) {
        err.errType = 'PARSE_ERROR';
        emitter.emit('error', ("PARSE_ERROR: '" + blueprintId + "'") + JSON.stringify(err));
        err.line = countLines(code, (ref = err.location[0]) != null ? ref.index : void 0);
        return cb(err);
      }
      return apiBlueprintAdapter.transform(result.ast, result.warnings || [], function(err, ast, warnings) {
        return cb(err, ast, warnings, result.sourcemap, result.ast);
      });
    });
  } else {
    try {
      apiaryAst = getAstSync(code);
    } catch (_error) {
      err = _error;
      return cb(err);
    }
    return cb(null, apiaryBlueprintAdapter.transform(apiaryAst));
  }
};

getAstSync = function(code) {
  return apiaryBlueprintParser.parse(code);
};

module.exports = {
  parseApiaryBlueprintSync: getAstSync,
  parse: getLocalAst,
  newVersionRegExp: NEW_VERSION_REGEXP
};
