// Generated by CoffeeScript 1.9.1
var Blueprint, CURRENT_AST_VERSION, JsonSchemaValidation, Request, Resource, Response, Section, combineParts, escapeBody, fillProps;

CURRENT_AST_VERSION = 17;

fillProps = function(object, props, defaults) {
  var key, ref, results;
  results = [];
  for (key in defaults) {
    results.push(object[key] = (ref = props[key]) != null ? ref : defaults[key]);
  }
  return results;
};

combineParts = function(separator, builder) {
  var parts;
  parts = [];
  builder(parts);
  return parts.join(separator);
};

escapeBody = function(body) {
  var i;
  if (/^>\s+|^<\s+|^\s*$/m.test(body)) {
    if (/^>>>\s*$/m.test(body)) {
      if (/^EOT$/m.test(body)) {
        i = 1;
        while (/^EOT#{i}$/m.test(body)) {
          i++;
        }
        return "<<<EOT" + i + "\n" + body + "\nEOT" + i;
      } else {
        return "<<<EOT\n" + body + "\nEOT";
      }
    } else {
      return "<<<\n" + body + "\n>>>";
    }
  } else {
    return body;
  }
};

Blueprint = (function() {
  Blueprint.fromJSON = function(json) {
    var s, v;
    if (json == null) {
      json = {};
    }
    return new this({
      location: json.location,
      name: json.name,
      metadata: json.metadata,
      version: json.version,
      description: json.description,
      htmlDescription: json.htmlDescription,
      sections: (function() {
        var j, len, ref, results;
        ref = json.sections || [];
        results = [];
        for (j = 0, len = ref.length; j < len; j++) {
          s = ref[j];
          results.push(Section.fromJSON(s));
        }
        return results;
      })(),
      validations: (function() {
        var j, len, ref, results;
        ref = json.validations || [];
        results = [];
        for (j = 0, len = ref.length; j < len; j++) {
          v = ref[j];
          results.push(JsonSchemaValidation.fromJSON(v));
        }
        return results;
      })()
    });
  };

  function Blueprint(props) {
    if (props == null) {
      props = {};
    }
    fillProps(this, props, {
      location: null,
      name: null,
      metadata: null,
      version: 1.0,
      description: null,
      htmlDescription: null,
      sections: [],
      validations: []
    });
  }

  Blueprint.prototype.resources = function(opts) {
    var j, k, len, len1, r, ref, ref1, resources, s;
    resources = [];
    ref = this.sections;
    for (j = 0, len = ref.length; j < len; j++) {
      s = ref[j];
      ref1 = s.resources;
      for (k = 0, len1 = ref1.length; k < len1; k++) {
        r = ref1[k];
        if ((opts != null ? opts.method : void 0) && opts.method !== r.method) {
          continue;
        }
        if ((opts != null ? opts.url : void 0) && opts.url !== r.url) {
          continue;
        }
        resources.push(r);
      }
    }
    return resources;
  };

  Blueprint.prototype.getUrlPrefixPosition = function() {
    var slashIndex, urlPrefixPosition, urlWithoutProtocol;
    urlPrefixPosition = 0;
    if (this.location) {
      urlWithoutProtocol = this.location.replace(/^https?:\/\//, "");
      slashIndex = urlWithoutProtocol.indexOf("/");
      if (slashIndex > 0) {
        urlPrefixPosition = urlWithoutProtocol.slice(slashIndex + 1).length;
      }
    }
    return urlPrefixPosition;
  };

  Blueprint.prototype.toJSON = function() {
    var s, urlPrefixPosition, v;
    urlPrefixPosition = this.getUrlPrefixPosition();
    return {
      location: this.location,
      name: this.name,
      metadata: this.metadata,
      version: this.version,
      description: this.description,
      htmlDescription: this.htmlDescription,
      sections: (function() {
        var j, len, ref, results;
        ref = this.sections;
        results = [];
        for (j = 0, len = ref.length; j < len; j++) {
          s = ref[j];
          results.push(s.toJSON(urlPrefixPosition));
        }
        return results;
      }).call(this),
      validations: (function() {
        var j, len, ref, results;
        ref = this.validations;
        results = [];
        for (j = 0, len = ref.length; j < len; j++) {
          v = ref[j];
          results.push(v.toJSON(urlPrefixPosition));
        }
        return results;
      }).call(this)
    };
  };

  Blueprint.prototype.toBlueprint = function() {
    var urlPrefixPosition;
    urlPrefixPosition = this.getUrlPrefixPosition();
    return combineParts("\n\n", (function(_this) {
      return function(parts) {
        var j, k, len, len1, ref, ref1, results, s, v;
        if (_this.location) {
          parts.push("HOST: " + _this.location);
        }
        if (_this.name) {
          parts.push("--- " + _this.name + " ---");
        }
        if (_this.description) {
          parts.push("---\n" + (("" + _this.description).trim()) + "\n---");
        }
        ref = _this.sections;
        for (j = 0, len = ref.length; j < len; j++) {
          s = ref[j];
          parts.push(s.toBlueprint(urlPrefixPosition));
        }
        if (_this.validations.length > 0) {
          parts.push("-- JSON Schema Validations --");
        }
        ref1 = _this.validations;
        results = [];
        for (k = 0, len1 = ref1.length; k < len1; k++) {
          v = ref1[k];
          results.push(parts.push(v.toBlueprint(urlPrefixPosition)));
        }
        return results;
      };
    })(this));
  };

  return Blueprint;

})();

Section = (function() {
  Section.fromJSON = function(json) {
    var r;
    if (json == null) {
      json = {};
    }
    return new this({
      name: json.name,
      description: json.description,
      htmlDescription: json.htmlDescription,
      resources: (function() {
        var j, len, ref, results;
        ref = json.resources || [];
        results = [];
        for (j = 0, len = ref.length; j < len; j++) {
          r = ref[j];
          results.push(Resource.fromJSON(r));
        }
        return results;
      })()
    });
  };

  function Section(props) {
    if (props == null) {
      props = {};
    }
    fillProps(this, props, {
      name: null,
      description: null,
      htmlDescription: null,
      resources: []
    });
  }

  Section.prototype.toJSON = function(urlPrefixPosition) {
    var r;
    return {
      name: this.name,
      description: this.description,
      htmlDescription: this.htmlDescription,
      resources: (function() {
        var j, len, ref, results;
        ref = this.resources || [];
        results = [];
        for (j = 0, len = ref.length; j < len; j++) {
          r = ref[j];
          results.push(r.toJSON(urlPrefixPosition));
        }
        return results;
      }).call(this)
    };
  };

  Section.prototype.toBlueprint = function(urlPrefixPosition) {
    return combineParts("\n\n", (function(_this) {
      return function(parts) {
        var j, len, r, ref, results;
        if (_this.name) {
          if (_this.description) {
            parts.push("--\n" + _this.name + "\n" + (("" + _this.description).trim()) + "\n--");
          } else {
            parts.push("-- " + _this.name + " --");
          }
        }
        ref = _this.resources || [];
        results = [];
        for (j = 0, len = ref.length; j < len; j++) {
          r = ref[j];
          results.push(parts.push((r.toBlueprint(urlPrefixPosition)) + "\n"));
        }
        return results;
      };
    })(this));
  };

  return Section;

})();

Resource = (function() {
  Resource.fromJSON = function(json) {
    var r;
    if (json == null) {
      json = {};
    }
    return new this({
      method: json.method,
      url: json.url,
      uriTemplate: json.uriTemplate,
      name: json.name,
      nameMethod: json.nameMethod,
      actionName: json.actionName,
      actionDescription: json.actionDescription,
      actionHtmlDescription: json.actionHtmlDescription,
      description: json.description,
      htmlDescription: json.htmlDescription,
      descriptionMethod: json.descriptionMethod,
      resourceDescription: json.resourceDescription,
      model: json.model,
      headers: json.headers,
      actionHeaders: json.actionHeaders,
      parameters: json.parameters,
      resourceParameters: json.resourceParameters,
      actionParameters: json.actionParameters,
      request: Request.fromJSON(json.request),
      requests: (function() {
        var j, len, ref, results;
        ref = json.requests || [];
        results = [];
        for (j = 0, len = ref.length; j < len; j++) {
          r = ref[j];
          results.push(Request.fromJSON(r));
        }
        return results;
      })(),
      responses: (function() {
        var j, len, ref, results;
        ref = json.responses || [];
        results = [];
        for (j = 0, len = ref.length; j < len; j++) {
          r = ref[j];
          results.push(Response.fromJSON(r));
        }
        return results;
      })()
    });
  };

  function Resource(props) {
    if (props == null) {
      props = {};
    }
    fillProps(this, props, {
      method: 'GET',
      url: '/',
      uriTemplate: '',
      name: void 0,
      nameMethod: void 0,
      actionName: void 0,
      actionDescription: void 0,
      actionHtmlDescription: void 0,
      description: void 0,
      htmlDescription: void 0,
      descriptionMethod: void 0,
      resourceDescription: void 0,
      model: void 0,
      headers: void 0,
      actionHeaders: void 0,
      parameters: void 0,
      resourceParameters: void 0,
      actionParameters: void 0,
      request: void 0,
      requests: [],
      responses: []
    });
  }

  Resource.prototype.getUrlFragment = function() {
    return (this.method.toLowerCase()) + "-" + (encodeURIComponent(this.url));
  };

  Resource.prototype.getExamples = function() {
    var examples, ids, j, k, key, len, len1, name, ref, ref1, reqOrResp;
    ids = [];
    examples = [];
    ref = ['requests', 'responses'];
    for (j = 0, len = ref.length; j < len; j++) {
      name = ref[j];
      ref1 = this[name] || [];
      for (k = 0, len1 = ref1.length; k < len1; k++) {
        reqOrResp = ref1[k];
        key = parseInt(reqOrResp.exampleId, 10) || 0;
        if (examples[key] == null) {
          examples[key] = {
            requests: [],
            responses: []
          };
        }
        examples[key][name].push(reqOrResp);
      }
    }
    return examples;
  };

  Resource.prototype.toJSON = function(urlPrefixPosition) {
    var r, ref;
    return {
      method: this.method,
      url: this.url,
      uriTemplate: this.uriTemplate || (urlPrefixPosition ? this.url.slice(urlPrefixPosition) : ''),
      name: this.name,
      nameMethod: this.nameMethod,
      actionName: this.actionName,
      actionDescription: this.actionDescription,
      actionHtmlDescription: this.actionHtmlDescription,
      description: this.description,
      htmlDescription: this.htmlDescription,
      descriptionMethod: this.descriptionMethod,
      resourceDescription: this.resourceDescription,
      model: this.model,
      headers: this.headers,
      actionHeaders: this.actionHeaders,
      parameters: this.parameters,
      resourceParameters: this.resourceParameters,
      actionParameters: this.actionParameters,
      request: (ref = this.request) != null ? ref.toJSON() : void 0,
      requests: (function() {
        var j, len, ref1, results;
        ref1 = this.requests || [];
        results = [];
        for (j = 0, len = ref1.length; j < len; j++) {
          r = ref1[j];
          results.push(r.toJSON());
        }
        return results;
      }).call(this),
      responses: (function() {
        var j, len, ref1, results;
        ref1 = this.responses || [];
        results = [];
        for (j = 0, len = ref1.length; j < len; j++) {
          r = ref1[j];
          results.push(r.toJSON());
        }
        return results;
      }).call(this)
    };
  };

  Resource.prototype.toBlueprint = function(urlPrefixPosition) {
    if (urlPrefixPosition == null) {
      urlPrefixPosition = 0;
    }
    return combineParts("\n", (function(_this) {
      return function(parts) {
        var ref, requestBlueprint, responsesBlueprint;
        if (_this.description) {
          parts.push(("" + _this.description).trim());
        }
        parts.push(_this.method + " " + (_this.url.slice(urlPrefixPosition)));
        requestBlueprint = (ref = _this.request) != null ? ref.toBlueprint() : void 0;
        if (requestBlueprint) {
          parts.push(requestBlueprint);
        }
        responsesBlueprint = combineParts("\n+++++\n", function(parts) {
          var j, len, r, ref1, results;
          ref1 = _this.responses || [];
          results = [];
          for (j = 0, len = ref1.length; j < len; j++) {
            r = ref1[j];
            results.push(parts.push(r.toBlueprint()));
          }
          return results;
        });
        if (responsesBlueprint) {
          return parts.push(responsesBlueprint);
        }
      };
    })(this));
  };

  return Resource;

})();

Request = (function() {
  Request.fromJSON = function(json) {
    if (json == null) {
      json = {};
    }
    return new this({
      name: json.name,
      description: json.description,
      htmlDescription: json.htmlDescription,
      headers: json.headers,
      reference: json.reference,
      body: json.body,
      schema: json.schema,
      exampleId: json.exampleId
    });
  };

  function Request(props) {
    if (props == null) {
      props = {};
    }
    fillProps(this, props, {
      name: void 0,
      description: void 0,
      htmlDescription: void 0,
      headers: {},
      reference: void 0,
      body: void 0,
      schema: void 0,
      exampleId: 0
    });
  }

  Request.prototype.toJSON = function() {
    return {
      name: this.name,
      description: this.description,
      htmlDescription: this.htmlDescription,
      headers: this.headers,
      reference: this.reference,
      body: this.body,
      schema: this.schema,
      exampleId: this.exampleId
    };
  };

  Request.prototype.toBlueprint = function() {
    return combineParts("\n", (function(_this) {
      return function(parts) {
        var name, ref, value;
        ref = _this.headers;
        for (name in ref) {
          value = ref[name];
          parts.push("> " + name + ": " + value);
        }
        if (_this.body) {
          return parts.push(escapeBody(_this.body));
        }
      };
    })(this));
  };

  return Request;

})();

Response = (function() {
  Response.fromJSON = function(json) {
    if (json == null) {
      json = {};
    }
    return new this({
      status: json.status,
      description: json.description,
      htmlDescription: json.htmlDescription,
      headers: json.headers,
      reference: json.reference,
      body: json.body,
      schema: json.schema,
      exampleId: json.exampleId
    });
  };

  function Response(props) {
    if (props == null) {
      props = {};
    }
    fillProps(this, props, {
      status: 200,
      description: void 0,
      htmlDescription: void 0,
      headers: {},
      reference: void 0,
      body: void 0,
      schema: void 0,
      exampleId: 0
    });
  }

  Response.prototype.toJSON = function() {
    return {
      status: this.status,
      description: this.description,
      htmlDescription: this.htmlDescription,
      headers: this.headers,
      reference: this.reference,
      body: this.body,
      schema: this.schema,
      exampleId: this.exampleId
    };
  };

  Response.prototype.toBlueprint = function() {
    return combineParts("\n", (function(_this) {
      return function(parts) {
        var name, ref, value;
        parts.push("< " + _this.status);
        ref = _this.headers;
        for (name in ref) {
          value = ref[name];
          parts.push("< " + name + ": " + value);
        }
        if (_this.body) {
          return parts.push(escapeBody(_this.body));
        }
      };
    })(this));
  };

  return Response;

})();

JsonSchemaValidation = (function() {
  JsonSchemaValidation.fromJSON = function(json) {
    if (json == null) {
      json = {};
    }
    return new this({
      status: json.status,
      method: json.method,
      url: json.url,
      body: json.body
    });
  };

  function JsonSchemaValidation(props) {
    if (props == null) {
      props = {};
    }
    fillProps(this, props, {
      status: void 0,
      method: "GET",
      url: "/",
      body: void 0
    });
  }

  JsonSchemaValidation.prototype.toJSON = function() {
    return {
      status: this.status,
      method: this.method,
      url: this.url,
      body: this.body
    };
  };

  JsonSchemaValidation.prototype.toBlueprint = function(urlPrefixPosition) {
    if (urlPrefixPosition == null) {
      urlPrefixPosition = 0;
    }
    return combineParts("\n", (function(_this) {
      return function(parts) {
        parts.push(_this.method + " " + (_this.url.slice(urlPrefixPosition)));
        if (_this.body) {
          return parts.push(escapeBody(_this.body));
        }
      };
    })(this));
  };

  return JsonSchemaValidation;

})();

module.exports = {
  Blueprint: Blueprint,
  Section: Section,
  Resource: Resource,
  Request: Request,
  Response: Response,
  Version: CURRENT_AST_VERSION
};
